//! @file $RCSfile: pfc_ctrl.c,v $
//!
//! Copyright (c) 2004 Atmel.
//!
//! Use of this program is subject to Atmel's End User License Agreement.
//! Please read file license.txt for copyright notice.
//!
//! @brief This file is a template for writing C software programs.
//!
//! This template file can be parsed by Doxygen for automatic documentation
//! generation.
//! Put here the functional description of this file within the software
//! architecture of your program.
//!
//! @version $Revision: 1.11 $ $Name: megaballast-refd-0_2_9 $ $Id: pfc_ctrl.c,v 1.11 2005/09/23 13:40:42 ebouin Exp $
//!
//! @todo
//! @bug

//_____  I N C L U D E S ___________________________________________________
#include "main_FBKIT.h"

//_____ M A C R O S ________________________________________________________

  //! @defgroup PFC_macros PFC Macros
  //! These macros allow to control the PFC
  //! @{

#define Get_v_haversine()   (U8) (get_adc_result(V_HAVERSINE))
#define Get_v_bus()         (U8) (get_adc_result(V_BUS))

  //! @}


//_____ D E F I N I T I O N S ______________________________________________

//! These three variables are timer variables which are incremented thanks
//! to the timer in main_LI100.c
extern U8 g_pfc_time_since_previous_timer_reset;
extern U8 g_time_since_previous_haversine_check;

extern U8 speed_mode;

extern volatile U8  gv_pfc_state;
extern volatile U8  gv_control_state;
extern volatile U8  gv_lamp_state;

extern volatile U8  gv_lamp_preset_current;

//! These two variables are volatile because they are both modified by code
//! and interrupt subroutine
volatile static U8 gvs_zcd_occures           = 0;
volatile static U8 gvs_pfc_soft_start_shots  = 0;

static U8 gs_v_haversine                     = 0;
static U8 gs_v_bus                           = 0;

static U8  gs_pfc_soft_start_tries           = 0;
static U8  gs_v_haversine_peak               = 0;
static U8  gs_v_haversine_nul                = 0;

#ifdef _LI100
static U8  gs_pfc_number_of_over_i_fault     = 0;
#endif //!< _LI100

static U8  gs_v_haversine_min_detected       = 0;
static U8  gs_v_haversine_min                = 255;
          

static U16 g_max_pulse_width                 = 0;

volatile static Psc_comparison_values gs_pfc_comparison_values;

static U8  gs_pfc_multiplier_time_since_previous_timer_reset = 0;
static S8  gs_error                                          = 0;

//static S8  gs_error_prec                                     = 0;

volatile static U8  gs_pfc_fault_state                       = 0;

//_____ D E C L A R A T I O N S ____________________________________________

//! This interrupt subroutine allows to know whether an ZCD interrupt
//! occured or not
//!
//! @warning Code:xx bytes (function code length)
//!
//! @param None
//!
//! @return None. Set gvs_zcd_occures to 1
//!

#ifdef _LI100

#pragma vector = PSC0_CAPT_vect
__interrupt void PSC0_CAPT_interrupt(void);

__interrupt void PSC0_CAPT_interrupt(void){
  if (Is_psc0_external_event_a_interrupt_flag_set()){
    Disable_both_psc2_outputs();
    Stop_psc2();
    gv_lamp_state = LAMP_OFF;
    gs_pfc_number_of_over_i_fault ++;
    gs_pfc_multiplier_time_since_previous_timer_reset = 0;
    g_pfc_time_since_previous_timer_reset = 0;
    Disable_psc0_external_event_a_interrupt();
    PFRC0A = 0x00;
    Clear_psc0_external_event_a_interrupt_flag();
    gs_pfc_fault_state = gv_pfc_state;
    //gv_pfc_state = PFC_OVER_I_FAULT;
    gv_pfc_state = PFC_PROBLEM;
  }else{ 
    //!< Is_psc0_external_event_b_interrupt_flag_set()
    //! A zero crossing detection occurs so The PFC is now started,
    //! Interrupts are no more needed
    gvs_zcd_occures = 1;
    Disable_psc0_external_event_b_interrupt();
  }
}
#endif //!< _LI100

#ifdef _FB_KIT
#pragma vector = ANACOMP_0_vect
__interrupt void ANACOMP_0_interrupt(void);

__interrupt void ANACOMP_0_interrupt(void){
   //! A zero crossing detection occurs so The PFC is now started,
   //! Interrupts are no more needed
   gvs_zcd_occures = 1;
   Disable_comparator0_interrupt();
}
#endif //!< _FB_KIT

//! This function retrurns the result of a AD converion on the wanted channel
//!
//! @warning Code:xx bytes (function code length)
//!
//! @param channel
//!
//! @return 8bits result.
//!
U8 get_adc_result(U8 channel){
  Start_conv_channel(channel);
  Wait_end_of_conversion();
  return (Adc_get_8_bits_result());
}

void update_pfc ( volatile Psc_comparison_values *psc0_comparison_values ){
    Lock_psc0_compare_values();
    OCR0SAH = HIGH(psc0_comparison_values->deadtime0);
    OCR0SAL = LOW (psc0_comparison_values->deadtime0);
    OCR0RAH = HIGH(psc0_comparison_values->ontime0);
    OCR0RAL = LOW (psc0_comparison_values->ontime0);
    OCR0SBH = HIGH(psc0_comparison_values->deadtime1);
    OCR0SBL = LOW (psc0_comparison_values->deadtime1);
    OCR0RBH = HIGH(psc0_comparison_values->ontime1);
    OCR0RBL = LOW (psc0_comparison_values->ontime1);
    Update_psc0_compare_values();
}
/*
void update_pfc_ontime0 ( volatile U16 ontime0 ){
    Lock_psc0_compare_values();
    OCR0RAH = HIGH(ontime0);
    OCR0RAL = LOW (ontime0);
    Update_psc0_compare_values();
}
*/
//! This function allows to control the PFC of the application.
//!
//! @warning Code:xx bytes (function code length)
//!
//! @param None
//!
//! @return None
//!
void pfc_task(){

 
  switch (gv_pfc_state) {

    case PFC_OFF: {
      if ( gv_lamp_preset_current != 0 ) { 
        gv_pfc_state = INIT_PFC;
      }
      break;
    } //!< END case PFC_OFF

    case INIT_PFC: {
#ifdef _LI100
      //! Activate the PFC overcurrent protection...
      //! Configure the analog_comparator
      //Set_comparator0_ni_vref_6_40(); //!< 400mV with 0.1 Ohms (4.0 Amps)
      Set_comparator0_ni_vref_1_60();   //!< 1,6 V with 0.2 Ohms (8.0 Amps)
      Enable_comparator0();
      //! Configure the fault mode
      PFRC0A = 0x67;
      Enable_psc0_external_event_a_interrupt();
      //! Fault mode OK   
#endif //!< _LI100
      gv_pfc_state = INIT_PFC_HAVERSINE_CHECK;
      break;
    } //!< END case INIT_PFC

    case INIT_PFC_HAVERSINE_CHECK: {
      //! Init the control values of the PFC.
      gs_v_haversine_peak = 0;
      gv_pfc_state = HAVERSINE_MEASURE;
      g_pfc_time_since_previous_timer_reset = 0;
      break;
    } //!< END case INIT_PFC_HAVERSINE_CHECK

    case HAVERSINE_MEASURE: {
      //! Measure the haversine peak voltage
      if (g_pfc_time_since_previous_timer_reset <= HAVERSINE_MIN_MEASURE_TIME){
        gs_v_haversine = (U8)(( (U16)(   (U8)(Get_v_haversine())
                                       + (U8)(gs_v_haversine)  ) ) >> 1 );
        //! Peak detection
        if (gs_v_haversine > gs_v_haversine_peak){
          gs_v_haversine_peak = gs_v_haversine;
        }
      }else{
        gv_pfc_state = HAVERSINE_CHECK;
      }
      break;
    } //!< END case HAVERSINE_MEASURE

    case HAVERSINE_CHECK: {
      //! PFC Haversine peak must be included between
      //! HAVERSINE_PEAK_MIN and HAVERSINE_PEAK_MAX (90VAC and 265VAC)
      if (  (gs_v_haversine_peak <= HAVERSINE_PEAK_MAX)
          &&(gs_v_haversine_peak >= HAVERSINE_PEAK_MIN) )
      {
        //! Set the max pulse width allowed
        g_max_pulse_width = (U8) (PFC_MAX_PULSE/gs_v_haversine_peak);//us
        if (g_max_pulse_width > (PFC_MAX_PULSE >> 6)){
          //! (PFC_MAX_PULSE >> 6) = PFC_MAX_PULSE at low speed it means 25uS
          g_max_pulse_width = (PFC_MAX_PULSE >> 6);
        }
        gs_pfc_soft_start_tries = 1;
        gv_pfc_state = CONFIGURE_PFC_SOFT_START;
      }else{
        gv_pfc_state = INIT_PFC_HAVERSINE_CHECK;
      }  
      break;
    } //!< END case HAVERSINE_CHECK

    case CONFIGURE_PFC_SOFT_START: {
      //! Configures the psc0  in order to make the PFC soft start
      //! PSC0 is not started until a Start_psc0() is executed
      Psc0_outputs_active_high();
      Disable_both_psc0_outputs();
      Psc0_in_4_ramps_mode();
      Disable_psc0_autolock_mode();
      Psc0_use_io_clock();
      Disable_psc0_prescaler();
      gvs_zcd_occures = 0;
      gs_pfc_comparison_values.deadtime0 = 0;
      gs_pfc_comparison_values.ontime0   = (g_max_pulse_width >> 1);
      gs_pfc_comparison_values.deadtime1 = 0;
      gs_pfc_comparison_values.ontime1   = PFC_START_PERIOD;
      update_pfc(&gs_pfc_comparison_values);

#ifdef _LI100
      //! Take care that in order to be able to generate an interrupt it is
      //! mandatory to use a fault mode... That's the reason why in reality
      //! The timing observed on the scope can take into acount the retriger
      //! action generated by the fault mode 8!
      PFRC0B = 0x88;
      Clear_psc0_external_event_b_interrupt_flag();
      Enable_psc0_external_event_b_interrupt();
      //! As soon as an interrupt will be generated, the interrupt subroutine
      //! will disable the interrupt event
#endif //!< _LI100

#ifdef _FB_KIT
      //! Configure the analog_comparator
      //Set_comparator0_ni_vref_6_40(); //!< 400mV
      Set_comparator0_ni_vref_3_20();   //!< 800mV
      Enable_comparator0_interrupt();
      Enable_comparator0();
      PFRC0B = 0x00;  // disable autoretrig
      Enable_psc0_end_of_cycle_interrupt();
#endif //!< _FB_KIT

      gv_pfc_state = PFC_SOFT_START;
      break;
    } //!< END case CONFIGURE_PFC_SOFT_START
      
    case PFC_SOFT_START: {
      //! Check that the soft start has been tried less than PFC_START_MAX_TRIES
      if (gs_pfc_soft_start_tries <= PFC_START_MAX_TRIES){
        gs_pfc_soft_start_tries ++;
        Enable_pscout00();
        Start_psc0();
        g_pfc_time_since_previous_timer_reset = 0;
        gv_pfc_state = SPEED_UP_MICROCONTROLLER;
      }else{
        gv_pfc_state = PFC_PROBLEM;
      }
      break;
    } //!< END case PFC_SOFT_START

    case SPEED_UP_MICROCONTROLLER: {
      if (Get_v_bus() >= V_BUS_OVERSHOOT){
        Disable_both_psc0_outputs();
        Stop_psc0();
        gs_pfc_multiplier_time_since_previous_timer_reset = 0;
        g_pfc_time_since_previous_timer_reset = 0;
        gv_pfc_state = PFC_DELAY_FOR_NEXT_SOFT_START;
      }else{
        if (gvs_zcd_occures){
          //! The PFC is now running, so the micro can run at its full speed
          //! and the lamp can be switched on
          //! Set the micro at its nominal speed
          Disable_all_it();
          CLKPR = 0x80;
          CLKPR = 0x00;
          Enable_all_it();
          TCCR1B = ( (0x18) | (TIMER_1_PRESCALER) );
          ICR1   = TIMER_1_TOP;
          TCNT1 = 0x00;
          //! Important to be sure the counter is not already higher than the
          //! comparison value... To catch the next interrupt
          
#ifdef _FB_KIT
          PFRC0B = 0x48;
#endif //!< _FB_KIT                      
          
          Stop_psc0();
          speed_mode = 1; // FAST;

#ifdef _DALI_CONTROL        
          dali_set_speed(speed_mode);
#endif //!< _DALI_CONTROL

          gvs_zcd_occures = 0;
          Set_adc_prescaler(2);
          //! Start the PLL for the PFC
          Start_pll_64_mega();
          Wait_pll_ready();
          Psc0_use_pll_clock();
          //! Set the appropriated values
          gs_v_haversine_nul = (U8)(0.3 * gs_v_haversine_peak);
          g_max_pulse_width = ( g_max_pulse_width * 64 );
          gs_pfc_comparison_values.deadtime0 = PFC_DEAD_TIME;
          gs_pfc_comparison_values.ontime0   = 0;
          gs_pfc_comparison_values.deadtime1 = 0;
          gs_pfc_comparison_values.ontime1   = PFC_MAX_PERIOD;
          update_pfc(&gs_pfc_comparison_values);
          Enable_pscout00();
          Start_psc0();
          //! Start the lamp control
          gv_pfc_state = PFC_FIND_ZCD;
          //! And finally... to be sure the PFC is updated ASAP
          g_pfc_time_since_previous_timer_reset = MAX_HALF_PERIOD;
        }else{
          //! There is a reason why it's not already time to speed_up the uc
          if (g_pfc_time_since_previous_timer_reset  >= 1){ 
            Disable_both_psc0_outputs();
            Stop_psc0();
            gs_pfc_multiplier_time_since_previous_timer_reset = 0;
            g_pfc_time_since_previous_timer_reset = 0;
            gv_pfc_state = PFC_DELAY_FOR_NEXT_SOFT_START;
          }
        }
      }
      break;
    } //!< END case SPEED_UP_MICROCONTROLLER

    case PFC_DELAY_FOR_NEXT_SOFT_START: {
    //! In case the soft start fails, the software has to wait
    //! DELAY_FOR_NEXT_PFC_SOFT_START*DELAY_MULTIPLIER_FOR_NEXT_PFC_SOFT_START
      //! (1S)before trying a new soft start
      if(g_pfc_time_since_previous_timer_reset > DELAY_FOR_NEXT_PFC_SOFT_START){
        gs_pfc_multiplier_time_since_previous_timer_reset ++;
        g_pfc_time_since_previous_timer_reset = 0;
        if (    gs_pfc_multiplier_time_since_previous_timer_reset
             >= DELAY_MULTIPLIER_FOR_NEXT_PFC_SOFT_START ){
          gv_pfc_state = CONFIGURE_PFC_SOFT_START;
        }
      }
      break;
    } //!< END case PFC_DELAY_FOR_NEXT_PFC_SOFT_START

    case PFC_FIND_ZCD:{
    //! Find the Zero Crossing Detection before updating the on time
      gs_v_haversine = (U8)(( (U16)((U8)(Get_v_haversine())
                                    + (U8)(gs_v_haversine)  ) ) >> 1 );
      if ( gs_v_haversine <= gs_v_haversine_nul){
        gs_pfc_multiplier_time_since_previous_timer_reset = 0;
        gv_pfc_state = PFC_CONTROL_LOOP;
      }
      break;
    }
    
    case PFC_CONTROL_LOOP: {
      //! The PFC is now running
      //! This is the normal pfc loop control...
      //! Measure the Bus voltage
      gs_v_bus = (U8)(( (U16)((U8)(Get_v_bus()) + (U8)(gs_v_bus)) ) >> 1 );
      if (gs_v_bus > V_BUS_OVERSHOOT){   //!< OVERSHOOT
        gs_pfc_comparison_values.ontime0 = 0;
      }else{                             //!< CORRECT VALUE
        //! Measure haversine voltage
        gs_v_haversine = (U8)(Get_v_haversine());
        gs_v_haversine = (U8)(( (U16)((U8)(Get_v_haversine())
                                    + (U8)(gs_v_haversine)  ) ) >> 1 );
        // Minimum detection
        if (g_pfc_time_since_previous_timer_reset >= EIGHT_MILLI_SECONDS){
          if (gs_v_haversine <= ( gs_v_haversine_min) ){
            gs_v_haversine_min = gs_v_haversine;
          }else{
            gs_v_haversine_min_detected = 1;
          }
        }
        if (gs_v_haversine_min_detected == 1){
          gs_v_haversine_min_detected = 0;
          gs_v_haversine_min = gs_v_haversine_peak;
          //! It's time to do it
          
          // Low cost PFC-P
          if ( (gs_v_bus) < (V_BUS_SET_POINT)){
            gs_pfc_comparison_values.ontime0 += ((V_BUS_SET_POINT - gs_v_bus)>>GAIN_PFC);  
          }else{
            gs_pfc_comparison_values.ontime0 -= ((gs_v_bus - V_BUS_SET_POINT)>>GAIN_PFC);  
          }
          
          /* PFC-P with two gains
          //! Calculate the error
          gs_error = V_BUS_SET_POINT - gs_v_bus;
          //! Calculate the new on_time
          if ( (gs_error > PFC_MARGE) || (gs_error < (-(2*PFC_MARGE))) ){
            //! P regulation with high gain
            gs_pfc_comparison_values.ontime0 = 
              (U16)(        (gs_pfc_comparison_values.ontime0)
                      +(S16)((gs_error)*(HIGH_PFC_GAIN))       );
          }else{
            //! P regulation with nominal gain
            gs_pfc_comparison_values.ontime0 = 
              (U16)(        (gs_pfc_comparison_values.ontime0)
                      +(S16)((gs_error)*(NOMINAL_PFC_GAIN))    );
          }
          */
          /*
          // PFC-PD
          //! Calculate the error
          gs_error = V_BUS_SET_POINT - gs_v_bus;
          gs_pfc_comparison_values.ontime0 = 
              (U16)(  (gs_pfc_comparison_values.ontime0) 
                    + ((gs_error) >> 1                 )
                    + ((gs_error - gs_error_prec) >> 4 )    );
          gs_error_prec = gs_error;
          */
          //  gs_pfc_comparison_values.ontime0 = 20;
          
          /* PFC-PID
          //! Calculate the error
          gs_error = V_BUS_SET_POINT - gs_v_bus;
          gs_pfc_comparison_values.ontime0 = 
            (U16)(   (gs_pfc_comparison_values.ontime0) 
                   + ((gs_error)                   >> 1 ) 
                   + ((gs_error - gs_error_prec)   >> 2 )
                   + ((gs_error + gs_error_prec)   >> 4 ) );
          gs_error_prec   = gs_error;
          */
 
          //! Correct the on_time value against absurd values
          if (gs_pfc_comparison_values.ontime0 > g_max_pulse_width) {
            if (gs_error > 0){
              gs_pfc_comparison_values.ontime0 = g_max_pulse_width;
            }else{
              gs_pfc_comparison_values.ontime0 = 0;
            }
          }
         //! The new value is now correct for sure
          gs_pfc_comparison_values.ontime1 = PFC_MAX_PERIOD;
          g_pfc_time_since_previous_timer_reset = 0;
          // update_pfc(&gs_pfc_comparison_values);
        }  
        update_pfc(&gs_pfc_comparison_values);
        //PORTD &= ~(1 << PIND4);
      }
      //! The wanted pulse width modulation is now programmed
      //! the PFC update will take place on the next PFC cycle
      break;
    } //!< END case PFC_CONTROL_LOOP
/*

#ifdef _LI100
    case PFC_OVER_I_FAULT: {
      //! In case the soft start fails, the software has to wait
      //! DELAY_FOR_NEXT_OVER_I_TRY * DELAY_MULTIPLIER_FOR_NEXT_OVER_I_TRY
      //! before directly going to PFC_CONTROL_LOOP
      //! Incase the problem appears more than MAX_OVER_I_FAULT the application
      //! is definitely stopped
      
      if (g_pfc_time_since_previous_timer_reset > DELAY_FOR_NEXT_OVER_I_TRY){
        gs_pfc_multiplier_time_since_previous_timer_reset ++;
        g_pfc_time_since_previous_timer_reset = 0;
        if (    gs_pfc_multiplier_time_since_previous_timer_reset
             >= DELAY_MULTIPLIER_FOR_NEXT_OVER_I_TRY )
        {
          if (    (gs_pfc_fault_state == PFC_CONTROL_LOOP)
               || (gs_pfc_fault_state == PFC_FIND_ZCD)     )
          {
            //! The uc is running at its nominal speed 
            if ( gs_pfc_number_of_over_i_fault < MAX_OVER_I_FAULTS ){
              PFRC0A = 0x67;
              Enable_psc0_external_event_a_interrupt();
              gs_pfc_comparison_values.deadtime0 = PFC_DEAD_TIME;
              gs_pfc_comparison_values.ontime0   = 0;
              gs_pfc_comparison_values.deadtime1 = 0;
              gs_pfc_comparison_values.ontime1   = PFC_MAX_PERIOD;
              update_pfc(&gs_pfc_comparison_values);
              Start_psc0();
              gv_pfc_state = PFC_FIND_ZCD;
            }else{
              gv_pfc_state = PFC_PROBLEM;
            }
          }else{
            gv_pfc_state = INIT_PFC;
          }
        }
      }
      break;
    } //!< END case PFC_OVER_I_FAULT
#endif //!< _LI100
*/    
    case SHUT_DOWN_PFC_AND_SLOW_DOWN_UC_SPEED: {
      //! Switch off the PFC
      Disable_both_psc0_outputs();
      Stop_psc0();
      Stop_pll();
      //! Switch the microcontroller to a low power consumption mode
      Disable_all_it();
      CLKPR = 0x80;
      CLKPR = 0x03;
      Enable_all_it();
      TCCR1B = ( (0x18) | (TIMER_1_PRESCALER_LOW_SPEED) );
      ICR1   = TIMER_1_TOP_LOW_SPEED;
      Set_adc_prescaler(1);
      speed_mode = 0; // SLOW;

#ifdef _DALI_CONTROL        
        dali_set_speed(speed_mode);
#endif //!< _DALI_CONTROL
      
      gv_pfc_state = PFC_OFF;
      break;
    } //!< END case SHUT_DOWN_PFC_AND_SLOW_DOWN_UC_SPEED

    case PFC_PROBLEM: {
      Disable_both_psc0_outputs();
      Stop_psc0();
      Stop_pll();
      Disable_adc();
      Disable_comparator0();
      //! Switch the microcontroller to a low power consumption mode
      Disable_all_it();
      CLKPR = 0x80;
      CLKPR = 0x03;
      Enable_all_it();
      gv_pfc_state = PFC_PROBLEM;
      break;
    } //!< END case PFC_PROBLEM

    default: {
      // BUG
      Disable_both_psc0_outputs();
      Stop_psc0();
      gv_pfc_state = INIT_PFC_HAVERSINE_CHECK;
      break;
    } //!< END case default

  } //!< End switch

} //!< End PFC_task
