//! @file $RCSfile: lamp_ctrl.c,v $
//!
//! Copyright (c) 2004 Atmel.
//!
//! Use of this program is subject to Atmel's End User License Agreement.
//! Please read file license.txt for copyright notice.
//!
//! @brief This file is a template for writing C software programs.
//!
//! This template file can be parsed by Doxygen for automatic documentation
//! generation.
//! Put here the functional description of this file within the software
//! architecture of your program.
//!
//! @version $Revision: 1.8 $ $Name: megaballast-refd-0_2_9 $ $Id: lamp_ctrl.c,v 1.8 2005/09/05 08:32:30 ebouin Exp $
//!
//! @todo
//! @bug

//_____  I N C L U D E S ___________________________________________________
#include "main_FBKIT.h"

//_____ M A C R O S ________________________________________________________

  //! @defgroup PFC_macros PFC Macros
  //! These macros allow to control the Lamp
  //! @{

#define Get_i_lamp()   (U8) (get_adc_result(I_LAMP))
#define Get_v_lamp()   (U8) (get_adc_result(V_LAMP))

  //! @}


      
//_____ D E F I N I T I O N S ______________________________________________

//! This variable is timer variable which is incremented thanks to the timer in main_megaballast_ref_d.c
extern U8 g_lamp_time_since_previous_timer_reset;
static U8 g_lamp_time_multiplier = 0;

extern volatile U8 gv_lamp_state;
extern volatile U8 gv_pfc_state;

extern volatile U8 gv_lamp_preset_current;

Psc_fifty_percent_comparison_values g_inverter_comparison_values;

static U8  gs_lamp_ignition_tries       = 0;
U8 g_number_of_lamps                    = 0;
U8 g_one_lamp_removed                   = 0;
U8 g_too_many_ignition_tries            = 0;
U8 g_no_lamp_default                    = 0;

//_____ D E C L A R A T I O N S ____________________________________________

void update_inverter_fifty ( Psc_fifty_percent_comparison_values *psc2_fifty_percent_comparison_values ){
  Lock_psc2_compare_values();
  OCR2SBH = HIGH(psc2_fifty_percent_comparison_values->deadtime1);
  OCR2SBL = LOW (psc2_fifty_percent_comparison_values->deadtime1);
  OCR2RBH = HIGH(psc2_fifty_percent_comparison_values->ontime1);
  OCR2RBL = LOW (psc2_fifty_percent_comparison_values->ontime1);
  Update_psc2_compare_values();
}

void lamp_task(){

static U8 gs_i_lamp = 0;
      
  switch (gv_lamp_state) {

    case LAMP_OFF: {
      if(    (gv_pfc_state == PFC_CONTROL_LOOP)
           &&((g_too_many_ignition_tries == 0)&&(g_no_lamp_default == 0)) )
      {
        gv_lamp_state = CONFIGURE_LAMP_PREHEAT;
      }
      break;
    }

    case CONFIGURE_LAMP_PREHEAT: {
      //! This is the first time the lamp is tried to be started since 
      //!    the user has request the switch on
      //! *** CONFIGURE THE AMPLIFIER 0 ***
      //! Amplifier0 is used to measure the current
      //! Its configuration never changes during the software execution
      Enable_amp0();
      Disable_amp0_input_shunt();
      Set_amp0_gain10();
      Amp0_internal_clock();
      //! *** CONFIGURE THE PSC2 ***
      //! PSC2 is not started until a Start_psc2() is executed
      Disable_both_psc2_outputs();
      Psc2_outputs_active_high();
      Psc2_in_4_ramps_mode();
      Enable_psc2_fifty_percent_mode();
      Disable_psc2_prescaler();
      Psc2_use_pll_clock();
      PSOC2 |= 0x20; //  Set the ADC conversion at the wanted moment
      g_inverter_comparison_values.deadtime1 = INVERTER_DEADTIME;
      g_inverter_comparison_values.ontime1   = INVERTER_START_PREHEAT_HALF_PERIOD;
      update_inverter_fifty(&g_inverter_comparison_values);
      Enable_both_psc2_outputs();
      Start_psc2();
      gs_lamp_ignition_tries = 0;
      g_lamp_time_since_previous_timer_reset = 0;
      g_lamp_time_multiplier = 0;
      gv_lamp_state = LAMP_PREHEAT;
      break;
    }
    
    case LAMP_PREHEAT: {
      //!< Let the preheat sequence for LAMP_PREHEAT_TIME
      if ( g_lamp_time_since_previous_timer_reset >= LAMP_PREHEAT_TIME){
        g_lamp_time_since_previous_timer_reset = 0;
        g_lamp_time_multiplier ++;
        if ( g_lamp_time_multiplier >= LAMP_PREHEAT_TIME_MULTIPLIER){
          gv_lamp_state = START_IGNITION;
          gs_lamp_ignition_tries ++;
          g_lamp_time_since_previous_timer_reset = 0;
        }
      }
      break;
    }

    case START_IGNITION: {
      //! Decrease the frequency from the init frequency down to INVERTER_IGNITION_HALF_PERIOD
      if (g_lamp_time_since_previous_timer_reset >= ONE_MILLI_SECOND){
        if (g_inverter_comparison_values.ontime1 < INVERTER_TWO_LAMP_IGNITION_HALF_PERIOD){
          g_inverter_comparison_values.ontime1 += IGNITION_STEP;
        }else{
          g_inverter_comparison_values.ontime1 = (INVERTER_TWO_LAMP_IGNITION_HALF_PERIOD);
          gv_lamp_state = IGNITION;
        }
        g_lamp_time_since_previous_timer_reset = 0;
        update_inverter_fifty(&g_inverter_comparison_values);
      }
      break;
    }

    case IGNITION: {
      //! The ignition sequence consists in maintaining the ignition frequency determined by INVERTER_IGNITION_HALF_PERIOD for 10ms
      //! Then in checking the ignition appears by measuring lamp current and voltage
      //! In case it is... START_RUN_MODE
      //! In case it isn't... RESTART_PREHEAT
      gs_i_lamp = Get_i_lamp();
      if (gs_i_lamp >= 0x80){
        gs_i_lamp = 0;
      } 
      if (g_lamp_time_since_previous_timer_reset >= LAMP_IGNITION_TIME){
        //! Check the ignition occurred
        gv_lamp_state = START_RUN_MODE;
        /*
          if (gs_i_lamp < TWO_LAMP_MINIMUM_IGNITION_CURRENT){
            gv_lamp_state = RESTART_PREHEAT;
          }
        */
        if (Get_v_lamp() >= IGNITION_MAXIMUM_IGNITION_VOLTAGE){
          gv_lamp_state = RESTART_PREHEAT;
        }
      }
      break;
    }

    case RESTART_PREHEAT: {
      //! Reconfigure the Inverter with the Restart parameters, then LAMP_PREHEAT
      //! If Ignition fails too many times... Stop!...
      if (gs_lamp_ignition_tries < LAMP_IGNITION_MAX_TRIES){
        g_inverter_comparison_values.deadtime1 = INVERTER_DEADTIME;
        g_inverter_comparison_values.ontime1   = INVERTER_RESTART_PREHEAT_HALF_PERIOD;
        update_inverter_fifty(&g_inverter_comparison_values);
        g_lamp_time_since_previous_timer_reset = 0;
        g_lamp_time_multiplier = 0;
        gv_lamp_state = LAMP_PREHEAT;
      }else{
        gv_lamp_state = TOO_MANY_LAMP_IGNITION_TRIES;
      }
      break;
    }

    case START_RUN_MODE: {
      //! Increase the frequency from the init frequency down to INVERTER_IGNITION_HALF_PERIOD
      if (g_lamp_time_since_previous_timer_reset >= ONE_MILLI_SECOND){
        if (g_inverter_comparison_values.ontime1 < INVERTER_RUN_HALF_PERIOD){
          g_inverter_comparison_values.ontime1 += IGNITION_STEP;
        }else{
          g_inverter_comparison_values.ontime1 = INVERTER_RUN_HALF_PERIOD;
          gv_lamp_state = RUN_MODE;
        }
        g_lamp_time_since_previous_timer_reset = 0;
        update_inverter_fifty(&g_inverter_comparison_values);
      }
      break;
    }

    case RUN_MODE: {
      //!< Normal control loop to have the light in accordance with the gv_lamp_preset_current
      gs_i_lamp = Get_i_lamp();
      if (gs_i_lamp >= 0x80){
        gs_i_lamp = 0;
      }  
      if (gs_i_lamp != gv_lamp_preset_current){
        if ( (gs_i_lamp) < (gv_lamp_preset_current)){
           g_inverter_comparison_values.ontime1 =
           (  (g_inverter_comparison_values.ontime1)
           +  (U8)((gv_lamp_preset_current-gs_i_lamp) >> GAIN_LAMP) ); //3
        }else{
           g_inverter_comparison_values.ontime1 =
           (  (g_inverter_comparison_values.ontime1)
           -  (U8)((gs_i_lamp-gv_lamp_preset_current) >> GAIN_LAMP) ); //3
        }
        if (g_inverter_comparison_values.ontime1 > INVERTER_MAX_PERIOD){
          g_inverter_comparison_values.ontime1 = INVERTER_MAX_PERIOD;
        }else{
          if(g_inverter_comparison_values.ontime1 < INVERTER_MIN_PERIOD){
             g_inverter_comparison_values.ontime1 = INVERTER_MIN_PERIOD;
          }
        }
        update_inverter_fifty(&g_inverter_comparison_values);
        g_lamp_time_since_previous_timer_reset = 0;
      }
//      if (Get_v_lamp() >= TWO_LAMP_MAXIMUM_RUNNING_VOLTAGE){
//        g_one_lamp_removed = 1;
//      }
      break;
    }

    case TOO_MANY_LAMP_IGNITION_TRIES :{
      g_too_many_ignition_tries = 1;
      break;
    }
/*
    case NO_LAMP: {
      g_no_lamp_default  = 1;
      // gv_lamp_on = 0;
      break;
    }
*/
    default: {
      // BUG
      break;
    }
  } // End switch
} // End lamp_task
