///! @file $RCSfile: lamp_ctrl.h,v $
//!
//! Copyright (c) 2004 Atmel.
//!
//! Please read file license.txt for copyright notice.
//!
//! @brief This file contains the low level macros and definition for the Lamp
//!
//! @version $Revision: 1.10 $ $Name: megaballast-refd-0_2_9 $
//!
//! @todo
//! @bug

#ifndef LAMP_LIB_H
#define LAMP_LIB_H

//_____ I N C L U D E S ____________________________________________________

//! @defgroup Lamp_module Lamp Module
//! Lamp Module
//! @{


//_____ M A C R O S ________________________________________________________



//_____ D E F I N I T I O N S ______________________________________________


#define FIVE_HUNDRED_NS              30 //!<  30   * 15,625 nS     = 468,75 nS deadtime

#define ONE_HUNDRED_AND_EIGHTY_KHZ  178 //!<  178 * 15,625 nS * 2 = 55.6   uS period (180   KHz)
#define ONE_HUNDRED_AND_SEVENTY_KHZ 188 //!<  188 * 15,625 nS * 2 = 58.75  uS period (170   KHz)

#define ONE_HUNDRED_AND_TWENTY_KHZ  266
#define ONE_HUNDRED_KHZ             320
#define EIGHTY_KHZ                  400 //!<  400 * 15,625 nS * 2 = 12.5   uS period (80   KHz)
#define SEVENTY_SEVEN_KHZ           415 //!<  415 * 15,625 nS * 2 = 12.969 uS period (77.1 KHz)
#define SEVENTY_KHZ                 457 //!<  457 * 15,625 nS * 2 = 14.281 uS period (70.0 KHz)
#define SIXTY_NINE_KHZ              464 //!<
#define SIXTY_EIGHT_KHZ             471 //!<
#define SIXTY_SEVEN_KHZ             478 //!<
#define SIXTY_SIX_KHZ               485 //!<
#define SIXTY_FIVE_KHZ              492 //!<  492 * 15,625 nS * 2 = 15.385 uS period (65.0 KHz)
#define SIXTY_FOUR_KHZ              500 //!<
#define SIXTY_THREE_KHZ             508 //!<
#define SIXTY_TWO_KHZ               516 //!<
#define SIXTY_ONE_KHZ               525 //!<
#define SIXTY_KHZ                   533 //!<  533 * 15,625 nS * 2 = 16.656 uS period (60.0 KHz)
#define FIFTY_NINE_KHZ              542 //!<
#define FIFTY_EIGHT_KHZ             552 //!<
#define FIFTY_SEVEN_KHZ             561 //!<
#define FIFTY_SIX_KHZ               571 //!<
#define FIFTY_FIVE_KHZ              582 //!<
#define FIFTY_FOUR_KHZ              593 //!<
#define FIFTY_THREE_KHZ             604 //!<
#define FIFTY_TWO_KHZ               615 //!<
#define FIFTY_ONE_KHZ               627 //!<
#define FIFTY_KHZ                   640 //!<  640 * 15.625 nS * 2 = 20     uS period (50   KHz)
#define FORTY_EIGHT_KHZ             667 //!<
#define FORTY_SIX_KHZ               696 //!<
#define FORTY_FIVE_KHZ              711 //!<
#define FORTY_KHZ                   800 //!<
#define THIRTY_FIVE_KHZ             914 //!<
#define THIRTY_TWO_KHZ             1000 //!< 1000 * 15,625 nS * 2 = 31.25  uS period (32.0 KHz)
#define THIRTY_KHZ                 1067 //!< 1067 * 15,625 nS * 2 = 33.33  uS period (30.0 KHz)
#define TWENTY_FIVE_KHZ            1280 //!< 1280 * 15,625 nS * 2 = 40.00  uS period (25.0 KHz)

#define INVERTER_MAX_PERIOD        THIRTY_KHZ      // 36W & 18W
#define INVERTER_MIN_PERIOD        ONE_HUNDRED_KHZ // 36W & 18W

   //! @defgroup Lamp_Init_values_definitions Lamp Init Values Definitions
   //! Give a name to Lamp inits values used into the program
   //! @{
#define INVERTER_GAIN 0.08//0.1

#define INVERTER_DEADTIME                       FIVE_HUNDRED_NS

#define INVERTER_START_PREHEAT_HALF_PERIOD      EIGHTY_KHZ        // 36W & 18W
#define INVERTER_RESTART_PREHEAT_HALF_PERIOD    SEVENTY_SEVEN_KHZ // 36W & 18W

#define INVERTER_TWO_LAMP_IGNITION_HALF_PERIOD  FORTY_KHZ         // 36W & 18W
#define INVERTER_ONE_LAMP_IGNITION_HALF_PERIOD  FORTY_FIVE_KHZ    // 36W & 18W


#define MINIMUM_TWO_LAMPS_CURRENT                15 //!< If at the end of the preheat phase the current is under this value, it means that the ballast has only one lamp
#define MINIMUM_ONE_LAMP_CURRENT                 10 //!< If at the end of the preheat phase the current is under this value, it means that the ballast has no lamp

#ifdef _36W_TUBES
#define     TWO_LAMP_MINIMUM_IGNITION_CURRENT       100 //!< During ignition, the current should be higher than 100 mA
#define     ONE_LAMP_MINIMUM_IGNITION_CURRENT        30 //!< During ignition, the current should be higher than  30 mA
#define     IGNITION_MAXIMUM_IGNITION_VOLTAGE        60 //!< During ignition, voltage should collapse under 600 mV
#define     MAX_TWO_LAMP_PRESET_CURRENT              80 //!< max running current mA
#define     MIN_TWO_LAMP_PRESET_CURRENT              10 //!< min running current mA
#define     INVERTER_RUN_HALF_PERIOD                SIXTY_KHZ
#define     GAIN_PFC                                  1
#define     GAIN_LAMP                                 4
#endif

#ifdef _18W_TUBES
#define     TWO_LAMP_MINIMUM_IGNITION_CURRENT        50 //!< During ignition, the current should be higher than 50 mA
#define     ONE_LAMP_MINIMUM_IGNITION_CURRENT        15 //!< During ignition, the current should be higher than 15 mA
#define     IGNITION_MAXIMUM_IGNITION_VOLTAGE        30 //!< During ignition, voltage should collapse under 300 mV
#define     MAX_TWO_LAMP_PRESET_CURRENT              40 //!< max running current mA
#define     MIN_TWO_LAMP_PRESET_CURRENT               1 //!< min running current mA
#define     INVERTER_RUN_HALF_PERIOD                 THIRTY_KHZ
#define     GAIN_PFC                                  1
#define     GAIN_LAMP                                 3
#endif


#define MAX_ONE_LAMP_PRESET_CURRENT (MAX_TWO_LAMP_PRESET_CURRENT >> 1)
#define MIN_ONE_LAMP_PRESET_CURRENT (MIN_TWO_LAMP_PRESET_CURRENT >> 1)


#define TWO_LAMP_MAXIMUM_RUNNING_VOLTAGE         27 //!< In case the measurement is lower than this value during running mode, assume that one lamp has been removed


#define LAMP_IGNITION_MAX_TRIES                   6 //!< Maximum ignition tries before default
#define MAX_LAMP_CURRENT_DEFAULT_HAPPENS          6 //!< Maximum overcurent times before shut dow

#define IGNITION_STEP                             1 //!< 15.625 * 2 = 31.25 nS/mS = 31.25 uS/S

#define LAMP_INVERTER_STEP                        1 //!< Inverter step to steady the control loop
   //! @}

   //! @defgroup Lamp_status_definitions Lamp status Definitions
   //! Lamp Status Definitions
   //! @{
#define LAMP_OFF                                   0 //!< The PFC is Off until the v_bus is within specification
#define CONFIGURE_LAMP_PREHEAT                     1
#define LAMP_PREHEAT                               2
#define START_IGNITION                             3
#define IGNITION                                   4
#define RESTART_PREHEAT                            5
#define START_RUN_MODE                             6
#define RUN_MODE                                   7
#define TOO_MANY_LAMP_IGNITION_TRIES               8
   //! @}

   //! @defgroup Lamp_times_definitions Lamp times Definitions
   //! Lamp Times Definitions
   //! @{
#define LAMP_PREHEAT_TIME                                TWENTY_MILLI_SECONDS
#define LAMP_PREHEAT_TIME_MULTIPLIER                     50 //! 20*50ms = 1s
#define LAMP_IGNITION_TIME                               TEN_MILLI_SECONDS
   //! @}

//_____ F U N C T I O N S __________________________________________________

void lamp_task(void);

//! @}
#endif  // LAMP_DRV_H
