///! @file $RCSfile: main_pwmx_fluo_demo.h,v $
//!
//! Copyright (c) 2004 Atmel.
//!
//! Please read file license.txt for copyright notice.
//!
//! @brief This file contains the low level macros and definition for the PLL
//!
//! @version $Revision: 1.10 $ $Name: megaballast-refd-0_2_9 $
//!
//! @todo
//! @bug

#ifndef REFD_H
#define REFD_H

//_____ I N C L U D E S ____________________________________________________

#define _18W_TUBES
#define _DALI_CONTROL
#define _SWITCH_CONTROL

//#define _LI100
#define _FB_KIT
//#define _DEBUG_SCHEDULER
//#define _DEBUG_SWITCH_CONTROL

#ifdef _LI100
#undef _FB_KIT
#endif

#ifdef _DALI_CONTROL
#undef _DEBUG_SCHEDULER
#undef _DEBUG_SWITCH_CONTROL
#endif //!< _DALI_CONTROL

#define FOSC_FAST                  8000000  //!< Oscillator Frequency : 8MHz
#define FOSC_SLOW                  1000000  //!< Oscillator Frequency : 8MHz

//! @defgroup Ref_Design_module  Reference Design Module
//! Reference Design Module
//! @{

#include "lib_mcu/compiler.h"
#include "lib_mcu/mcu.h"
#include "lib_mcu/adc/adc_drv.h"
#include "lib_mcu/amplifier/amplifier_drv.h"
#include "lib_mcu/comparator/comparator_drv.h"
#include "lib_mcu/psc/psc_drv.h"
#include "lib_mcu/pll/pll_drv.h"

#include "lib_mcu/dali/dali_config.h"
#include "lib_mcu/dali/dali_lib.h"
#include "lib_mcu/dali/eeprom.h"

#include "lamp_FBKIT.h"
#include "pfc_FBKIT.h"
#include "control_FBKIT.h"


//_____ M A C R O S ________________________________________________________

//_____ D E F I N I T I O N S ______________________________________________



//! @defgroup Reference_design_definitions Reference Design Definitions
//! Reference Design definitions
//! @{

   //! @defgroup Adc_channel_definitions Channel Name Definitions
   //! Give a name to the ADC channels
   //! @{

#define V_HAVERSINE           4  //!< PB7 : ADC4       PIN24   Haversine Voltage Measurement
#define TEMPERATURE           6  //!< PB5 : ADC6       PIN22   Temperature sensor measurement
#define I_LAMP               11  //!< PB4 : AMP0+      PIN21   Lamp Current Measurement
#define V_LAMP                3  //!< PD6 : ADC3       PIN14   Lamp voltage Measurement
#define V_BUS                 5  //!< PB2 : ADC5       PIN16   Bus Voltage Measurement

   //! @}

   //! @defgroup Reference_design_timer_definitions Reference Design Timer Definitions
   //! Reference Design Timer Definitions
   //! @{

#define TICK_DIVISION                  10 //!< 1mS/100uS = 10 
#define LOW_SPEED_TICK_DIVISION         2 //!< 1mS/500uS =  2 

#define TIMER_1_PRESCALER_LOW_SPEED     2 //!< The prescaler of the timer 1 will be configured in order to divide the I/O clock by 8 for starting
#define TIMER_1_TOP_LOW_SPEED          62 //!< The timer 1 will generate a clock event each 62 * (64*125ns) = 500uS
#define TIMER_1_PRESCALER               3 //!< The prescaler of the timer 1 will be configured in order to divide the I/O clock by 64
#define TIMER_1_TOP                     12//!< The timer 1 will generate a clock event each 12,5 * (64*125ns) = 100uS
#define ONE_MILLI_SECOND                1 * TICK_DIVISION
#define TWO_MILLI_SECONDS               2 * TICK_DIVISION
#define THREE_MILLI_SECONDS             3 * TICK_DIVISION
#define FOUR_MILLI_SECONDS              4 * TICK_DIVISION
#define FIVE_MILLI_SECONDS              5 * TICK_DIVISION
#define SIX_MILLI_SECONDS               6 * TICK_DIVISION
#define EIGHT_MILLI_SECONDS             8 * TICK_DIVISION
#define TEN_MILLI_SECONDS              10 * TICK_DIVISION
#define TWENTY_MILLI_SECONDS           20 * TICK_DIVISION

#define LOW_SPEED_ONE_MILLI_SECOND      1 * LOW_SPEED_TICK_DIVISION
#define LOW_SPEED_TWO_MILLI_SECONDS     2 * LOW_SPEED_TICK_DIVISION
#define LOW_SPEED_THREE_MILLI_SECONDS   3 * LOW_SPEED_TICK_DIVISION
#define LOW_SPEED_FOUR_MILLI_SECONDS    4 * LOW_SPEED_TICK_DIVISION
#define LOW_SPEED_FIVE_MILLI_SECONDS    5 * LOW_SPEED_TICK_DIVISION
#define LOW_SPEED_TEN_MILLI_SECONDS    10 * LOW_SPEED_TICK_DIVISION
#define LOW_SPEED_TWENTY_MILLI_SECONDS 20 * LOW_SPEED_TICK_DIVISION


#define TIME_TO_WAIT_BETWEEN_TWO_CONTROL_TASKS       TWO_MILLI_SECONDS
#define LOW_TIME_TO_WAIT_BETWEEN_TWO_CONTROL_TASKS   LOW_SPEED_TWO_MILLI_SECONDS
   //! @}

//! @}

//_____ D E C L A R A T I O N S ____________________________________________

#endif /* REF_D_H */
