///! @file $RCSfile: pfc_ctrl.h,v $
//!
//! Copyright (c) 2004 Atmel.
//!
//! Please read file license.txt for copyright notice.
//!
//! @brief This file contains the low level macros and definition for the PFC
//!
//! @version $Revision: 1.11 $ $Name:  $
//!
//! @todo
//! @bug

#ifndef PFC_LIB_H
#define PFC_LIB_H

//_____ I N C L U D E S ____________________________________________________

//! @defgroup PFC_module PFC Module
//! PFC Module
//! @{


//_____ M A C R O S ________________________________________________________



//_____ D E F I N I T I O N S ______________________________________________

   //! @defgroup PFC_Init_values_definitions PFC Init Values Definitions
   //! Give a name to PFC inits values used into the program
   //! @{

#define HAVERSINE_PEAK_MAX               138 //!< 250* 10 = 2497mV (106 * 2,500 = 265V)
#define HAVERSINE_PEAK_MIN                69 //!< 85 * 10 =  848mV (106 * 0,850 = 90,1V)
#define HAVERSINE_MIN_MEASURE_TIME       LOW_SPEED_FIVE_MILLI_SECONDS

#define LOW_PFC_FREQUENCY_MULTIPLIER       8 // 16

#define PFC_START_PULSE                   10 //!< TEN_US with CKDIV8
#define PFC_START_PERIOD                  40 //!< FIFTY_US with CKDIV8
#define PFC_DEAD_TIME                     13 //!< ONE_US = 64
#define PFC_MAX_PERIOD                  2560 //!< (40*64) // FORTY_US
#define PFC_MAX_PULSE                   1600 //!< (25*64) // TWENTY_FIVE_US Highest pulse applicable

#define PFC_START_MAX_TRIES               10 //!< number of soft start tries before considering there is a problem

#define MAX_OVER_I_FAULTS                 10

#define V_BUS_OVERSHOOT                  254 //!< V_BUS max value (415VDC)
#define V_BUS_SET_POINT                  239 //!< V_BUS set point voltage (400VDC)

#define MIN_HALF_PERIOD                   90 //!< 100uS * 10 *  9 =   9 mS
#define MAX_HALF_PERIOD                  110 //!< 100uS * 10 * 11 =  11 mS


   //! @}

   //! @defgroup PFC_status_definitions PFC status Definitions
   //! PFC Status Definitions
   //! @{

#define PFC_OFF                                  0
#define INIT_PFC                                 1
#define INIT_PFC_HAVERSINE_CHECK                 2
#define HAVERSINE_MEASURE                        3
#define HAVERSINE_CHECK                          4
#define CONFIGURE_PFC_SOFT_START                 5
#define PFC_SOFT_START                           6
#define SPEED_UP_MICROCONTROLLER                 7
#define PFC_DELAY_FOR_NEXT_SOFT_START            8
#define PFC_FIND_ZCD                             9
#define PFC_CONTROL_LOOP                        10
#define PFC_OVER_I_FAULT                        11
#define SHUT_DOWN_PFC_AND_SLOW_DOWN_UC_SPEED    12
#define PFC_PROBLEM                             13

   //! @}

   //! @defgroup PFC_ticks_definitions PFC ticks Definitions
   //! PFC Ticks Definitions
   //! @{
#define DELAY_FOR_NEXT_PFC_SOFT_START                  LOW_SPEED_TWENTY_MILLI_SECONDS
#define DELAY_MULTIPLIER_FOR_NEXT_PFC_SOFT_START       50
#define DELAY_FOR_NEXT_OVER_I_TRY                      TWENTY_MILLI_SECONDS
#define DELAY_MULTIPLIER_FOR_NEXT_OVER_I_TRY           250
  //! @}

//_____ F U N C T I O N S __________________________________________________

U8 get_adc_result(U8 channel);
void pfc_task(void);

//! @}
#endif  // PFC_DRV_H
